/* ----------------------------------------------------------------------------
 * file: DN5Test.java
 * project: ConfigDB-Persistence
 * by: Marc Lewerentz
 *
 * Copyright (c) 2016 - 2016, All rights reserved.
 * Max-Planck-Institut für Plasmaphysik. W7-X CoDaC group.
 * ----------------------------------------------------------------------------
 */
package de.mpg.ipp.codac.persistence;

import java.util.List;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.jdo.Transaction;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.mpg.ipp.codac.business.schema.testcase.TestUser;
import de.mpg.ipp.codac.business.schema.testcase.TestUserGroup;
import de.mpg.ipp.codac.common.PersistenceProvider;

public class DN5Test {

    private static final int access = PersistenceProvider.READ_WRITE;
    private static final String uid = "marcl";
    private static DataNucleusConfigProvider provider;

    @BeforeClass
    public static void setUpBeforeClass()
        throws Exception {

        provider = new DNOptimizedMongoDBConfigProvider("mongodb:localhost/TestDB", access, uid);//please establish connection to your test database or mockup objects
        Assert.assertNotNull(provider);
    }

    @AfterClass
    public static void tearDownAfterClass()
        throws Exception {
        if (provider != null)
            provider.close();
    }

    @Before
    public void setUp()
        throws Exception {}

    @After
    public void tearDown()
        throws Exception {}

    @SuppressWarnings("deprecation")
    @Test
    public void testReadUserGroup() {

        TestUser user = new TestUser();
        TestUser[] users = { user };
        TestUserGroup userGroup = new TestUserGroup();
        userGroup.setUsers(users);
        System.out.println(userGroup);
        TestUserGroup storedUserGroup;
        TestUserGroup readUserGroup = null;

        PersistenceManager pm = provider.getPersistenceManager(); //please use your way of getting a persistence manager

        Transaction tx = pm.currentTransaction();

        try {
            tx.begin();

            storedUserGroup = pm.makePersistent(userGroup);

            tx.commit();
        } finally {

            if (tx.isActive())
                tx.rollback();

            pm.close();
        }

        System.out.println(user);
        System.out.println(storedUserGroup);

        pm = provider.getPersistenceManager(); //please use your way of getting a persistence manager
        tx = pm.currentTransaction();

        try {
            tx.begin();
            Query<?> q = null;
            pm.getFetchPlan().addGroup("deep");
            q =
                pm.newQuery("SELECT FROM de.mpg.ipp.codac.business.schema.testcase.TestUserGroup WHERE sysId == "
                            + storedUserGroup.getSysId());

            List<?> c = (List<?>) q.execute();
            if (c != null && !c.isEmpty())
                readUserGroup = (TestUserGroup) c.get(0);

            tx.commit();

        } finally {

            if (tx.isActive())
                tx.rollback();

            pm.close();
        }

        Assert.assertNotNull(readUserGroup);
        System.out.println(readUserGroup);
    }

}
